package spi_slave::av_st_sink;
use europa_module_factory;
@ISA = qw(europa_module_factory);

use strict;

sub add_contents_to_module
{
  my $this = shift;
  my $module = $this->module();

  # Avalon-ST sink.  This module doesn't have much reason to exist -
  # it's just wires.  But perhaps it makes for a cleaner block structure.
  $module->add_contents(
    e_port->new({name => "clk", direction => 1,}),
    e_port->new({name => "reset_n", direction => 1,}),
    e_assign->new({
      lhs => {name => "tx_fifo_writedata", width => $this->datawidth(),},
      rhs => {name => "sink_data", width => $this->datawidth(),},
    }),
    e_assign->new({
      lhs => "tx_fifo_write",
      rhs => "sink_valid",
    }),
    e_assign->new({
      lhs => "sink_ready",
      rhs => "~tx_fifo_full",
    }),
  );
}

sub get_fields
{
  my $class = shift;

  my %fields = (
    datawidth => { range => [1, undef], },
  );

  return \%fields;
}

#  st sink is like a writeable slave
#  valid -> write       (input)
#  data  -> writedata   (input)
#  ready -> waitrequest (output)
# 
#              clk  
#          reset_n
#       sink_valid ---------- tx_fifo_write
#        sink_data ---------- tx_fifo_writedata
#    ~tx_fifo_full ---------- sink_ready

sub declare_ports
{
  my $this = shift;

  my %inputs = (
    clk => {
      width => 1,
    },
    reset_n => {
      width => 1,
    },
    sink_valid => {
      width => 1,
    },
    sink_data => {
      width => $this->datawidth(),
    },
    tx_fifo_full => {
      width => 1,
    },
  );
  my %outputs = (
    sink_ready => {
      width => 1,
    },
    tx_fifo_write => {
      width => 1,
    },
    tx_fifo_writedata => {
      width => $this->datawidth(),
    },
  );
  map {$inputs{$_}->{direction} = "input"} keys %inputs;
  map {$outputs{$_}->{direction} = "output"} keys %outputs;
  my %ports = (
    %inputs,
    %outputs,
  );

  return \%ports;
}

1;

