package spi_slave::av_st_source;
use europa_module_factory;
@ISA = qw(europa_module_factory);

use strict;

sub add_contents_to_module
{
  my $this = shift;
  my $module = $this->module();

  # Avalon-ST source.  This module doesn't have much reason to exist -
  # it's just wires.  But perhaps it makes for a cleaner block structure.
  $module->add_contents(
    e_port->new({name => "clk", direction => 1,}),
    e_port->new({name => "reset_n", direction => 1,}),
    e_assign->new({
      lhs => {name => "source_data", width => $this->datawidth(),},
      rhs => {name => "rx_fifo_readdata", width => $this->datawidth(),},
    }),
    e_assign->new({
      lhs => "rx_fifo_read",
      rhs => "source_ready & ~rx_fifo_empty",
    }),
    e_assign->new({
      lhs => "source_valid",
      rhs => "~rx_fifo_empty",
    }),
  );
}

sub get_fields
{
  my $class = shift;

  my %fields = (
    datawidth => { range => [1, undef], },
  );

  return \%fields;
}

#
#  st source is like a writing master
#  valid -> write        (output)
#  data  -> writedata    (output)
#  ready -> waitrequest  (input)
# 
#              clk  
#          reset_n
#     source_ready ---------- rx_fifo_read
# rx_fifo_readdata ---------- source_data
#   ~rx_fifo_empty ---------- source_valid
# 
sub declare_ports
{
  my $this = shift;

  my %inputs = (
    clk => {
      width => 1,
    },
    reset_n => {
      width => 1,
    },
    source_ready => {
      width => 1,
    },
    rx_fifo_empty => {
      width => 1,
    },
    rx_fifo_readdata => {
      width => $this->datawidth(),
    },
  );
  my %outputs = (
    source_data => {
      width => $this->datawidth(),
    },
    source_valid => {
      width => 1,
    },
    rx_fifo_read => {
      width => 1,
    },
  );
  map {$inputs{$_}->{direction} = "input"} keys %inputs;
  map {$outputs{$_}->{direction} = "output"} keys %outputs;
  my %ports = (
    %inputs,
    %outputs,
  );

  return \%ports;
}

1;

