package spi_slave::control;
use europa_utils qw(ceil log2);
use europa_module_factory;
@ISA = qw(europa_module_factory);

use strict;

sub add_contents_to_module
{
  my $this = shift;
  my $module = $this->module();

  my $dw = $this->datawidth();

  my $width = europa_utils::ceil(europa_utils::log2($dw));
  my $reset_val = "$width\'b0";
  my $terminal_count = sprintf("%d'd%d", $width, $this->datawidth() - 1);

  $module->add_contents(
    e_register->new({
      enable => "1'b1",
      in => "sync_SCLK",
      out => "d1_sync_SCLK",
    }),
    e_assign->new({
      comment => "Rising SCLK: sample",
      lhs => {name => "sample",},
      rhs => "~d1_sync_SCLK & sync_SCLK",
    }),
    e_assign->new({
      comment => "Falling SCLK: shift",
      lhs => {name => "shift", export => 1,},
      rhs => "d1_sync_SCLK & ~sync_SCLK",
    }),
  );

  # The counter has to roll over to 0 after the terminal count; can't
  # rely on SS_n for a reset.
  my $incremented_val = 
    "((count < $terminal_count) ? (count + 1'b1) : $reset_val)";
  $module->add_contents(
    e_assign->new({
      lhs => {name => "p1_count", width => $width,},
      rhs => "sync_SS_n ? $reset_val : shift ? $incremented_val : count",
    }),
    e_register->new({
      out => "d1_shift", 
      in => "shift",
      enable => "1'b1",
    }),
    e_register->new({
      async_value => $reset_val,
      out => {name => "count", width => $width,},
      in => "p1_count",
      enable => "1'b1",
    }),
    # fifo write must be based on d1_shift because fifo write data
    # loads into its shift register on shift, so isn't ready for fifo
    # write for an additional cycle.  Might be easy to save a cycle
    # by driving rxbit directly to the rx lsb/msb, so it's available
    # after sample, then can be written to fifo upon shift (as well as
    # shifting into the shift register).
    e_assign->new({
      lhs => "rx_fifo_write",
      rhs => "d1_shift & (count == $reset_val)",
    }),
    e_assign->new({
      lhs => {name => "txshift_load", export => 1,},
      rhs => "(count == $reset_val) & sample",
    }),
     e_assign->new({
      lhs => "tx_fifo_read",
      rhs => "txshift_load",
    }),
  );
}

sub get_fields
{
  my $class = shift;

  my %fields = (
    datawidth => { range => [1, undef], },
  );

  return \%fields;
}

1;

