package spi_slave::fifo;
use europa_module_factory;
@ISA = qw(europa_module_factory);
use strict;

sub add_contents_to_module
{
  my $this = shift;
  my $module = $this->module();

  my $dw = $this->datawidth();

  $module->add_contents(
    e_register->new({
      out => {name => "readdata", width => $dw,},
      in => {name => "writedata", width => $dw,},
      enable => "write",
    }),
    #  empty  read  write  p1_empty
    #      0     0      0         0
    #      0     0      1         0 (overflow)
    #      0     1      0         1
    #      0     1      1         0
    #      1     0      0         1
    #      1     0      1         0
    #      1     1      0         1 (underflow)
    #      1     1      1         0 (underflow)
    e_assign->new({
      lhs => "p1_empty",
      rhs => "(read & ~write) | (empty & ~read & ~write)",
    }),
    e_register->new({
      out => {name => "empty", export => 1,},
      in => "p1_empty",
      enable => "1'b1",
      async_value => "1'b1",
    }),
    #   full  read  write  p1_full
    #      0     0      0        0
    #      0     0      1        1
    #      0     1      0        0 (underflow)
    #      0     1      1        1 (underflow)
    #      1     0      0        1
    #      1     0      1        1 (overflow)
    #      1     1      0        0
    #      1     1      1        1

    # Naturally (since I have only a single storage location),
    # full = ~empty.  I'll encode that fact in a slightly more
    # elaborate way, to preserve the sort of structure I'm going
    # to need if/when this FIFO ever has more than one location.
    e_register->new({
      out => {name => "full", export => 1,},
      in => "~p1_empty",
      enable => "1'b1",
      async_value => "1'b0",
    }),
  );
}

sub get_fields
{
  my $class = shift;

  my %fields = (
    datawidth => { range => [1, undef], },
    depth => { allowed_values => [1], },
  );

  return \%fields;
}

sub declare_ports
{
  my $this = shift;

  my %inputs = (
    clk => {
      width => 1,
    },
    reset_n => {
      width => 1,
    },
    read => {
      width => 1,
    },
    write => {
      width => 1,
    },
    writedata => {
      width => $this->datawidth(),
    },
  );
  my %outputs = (
    full => {
      width => 1,
    },
    empty => {
      width => 1,
    },
    readdata => {
      width => $this->datawidth(),
    },
  );
  map {$inputs{$_}->{direction} = "input"} keys %inputs;
  map {$outputs{$_}->{direction} = "output"} keys %outputs;
  my %ports = (
    %inputs,
    %outputs,
  );

  return \%ports;
}

1;

