package spi_slave::rx;
use europa_module_factory;
@ISA = qw(europa_module_factory);

use strict;

sub add_contents_to_module
{
  my $this = shift;
  my $module = $this->module();

  my $dw = $this->datawidth();

  my $rxshift_expression;
  if ($this->lsbfirst())
  {
    my $msb = $dw - 1;
    $rxshift_expression = "{sync_MOSI, rxshift[$msb : 1]}";
  }
  else
  {
    my $msb2 = $dw - 2;
    $rxshift_expression = "{rxshift[$msb2 : 0], sync_MOSI}";
  }

  $module->add_contents(
    e_register->new({
      out => {name => "rxshift", width => $dw,},
      in => $rxshift_expression,
      enable => "shift",
    }),
    e_assign->new({
      lhs => {name => "rx_fifo_writedata", width => $this->datawidth(),},
      rhs => "rxshift",
    })
  );
}

sub get_fields
{
  my $class = shift;

  my %fields = (
    datawidth => 
    {
      range => [1, undef],
    },
    lsbfirst => 
    {
      range => [0, 1],
    },
  );

  return \%fields;
}

sub declare_ports
{
  my $this = shift;

  my %inputs = (
    clk => {
      width => 1,
    },
    reset_n => {
      width => 1,
    },
    sync_MOSI => {
      width => 1,
    },
    shift => {
      width => 1,
    },
  );
  my %outputs = (
    rx_fifo_writedata => {
      width => $this->datawidth(),
    },
  );

  map {$inputs{$_}->{direction} = "input"} keys %inputs;
  map {$outputs{$_}->{direction} = "output"} keys %outputs;

  my %ports = (
    %inputs,
    %outputs,
  );

  return \%ports;

}

1;

