package spi_slave::spi_slave_mm;
use europa_module_factory;
@ISA = qw(europa_module_factory);
use spi_slave::spi_slave_st;
use strict;

sub add_contents_to_module
{
  my $this = shift;
  my $module = $this->module();

  my $st = spi_slave::spi_slave_st->new({
    lsbfirst => $this->lsbfirst(),
    datawidth => $this->datawidth(),
  });
  $module->add_contents(
    e_signal->new({name => "writedata", width => $this->datawidth(),}),
    e_signal->new({name => "readdata", width => $this->datawidth(),}),
    $st->make_instance({
      port_map => {
        sink_data => "writedata",
        sink_valid => "write",
        source_data => "readdata",
        source_ready => "read",
        sink_ready => "open",
      },
    }),
   # Note that source_valid implies sink_ready...
    e_assign->new({
      lhs => "waitrequest",
      rhs => "(read | write) & ~source_valid",
    }),
  );
}

# Note: static method; can be called for constructing help text.
sub get_fields
{
  my $class = shift;

  my %fields = (
    datawidth => { 
      range => [1, undef], 
      description => "Data width",
    },
    lsbfirst => { 
      range => [0, 1], 
      description => "data direction (0: msb first; 1: lsb first)",
    },
  );

  return \%fields;
}

sub declare_ports
{
  my $this = shift;

  my %inputs = (
    # Standard clock/reset
    clk => {
      width => 1,
    },
    reset_n => {
      width => 1,
    },

    # SPI inputs
    SS_n => {
      width => 1,
    },
    MOSI => {
      width => 1,
    },
    SCLK => {
      width => 1,
    },

    # Avalon-MM slave inputs
    writedata => {
      width => $this->datawidth(),
    },
    write => {
      width => 1,
    },
    read => {
      width => 1,
    },
  );
  my %outputs = (
    # SPI output
    MISO => {
      width => 1,
    },
    # Avalon-MM slave outputs
    readdata => {
      width => $this->datawidth(),
    },
    waitrequest => {
      width => 1,
    },
  );
  map {$inputs{$_}->{direction} = "input"} keys %inputs;
  map {$outputs{$_}->{direction} = "output"} keys %outputs;
  my %ports = (
    %inputs,
    %outputs,
  );

  return \%ports;
}

1;

