package spi_slave::spi_slave_st;
use europa_module_factory;
@ISA = qw(europa_module_factory);
use spi_slave::rx;
use spi_slave::tx;
use spi_slave::sync;
use spi_slave::fifo;
use spi_slave::control;
use spi_slave::av_st_sink;
use spi_slave::av_st_source;
use strict;

sub add_contents_to_module
{
  my $this = shift;
  my $module = $this->module();

  my $rx = spi_slave::rx->new({
    lsbfirst => $this->lsbfirst(),
    datawidth => $this->datawidth(),
  });
  $module->add_contents(
    $rx->make_instance({
      name => "the_rx",
    }),
  );

  my $tx = spi_slave::tx->new({
    lsbfirst => $this->lsbfirst(),
    datawidth => $this->datawidth(),
  });
  $module->add_contents(
    $tx->make_instance({
      name => "the_tx",
    }),
  );

  my $fifo = spi_slave::fifo->new({
    datawidth => $this->datawidth(),
    depth => 1,
  });
  $module->add_contents(
    $fifo->make_instance({
      name => "rx_fifo",
      port_map => {
         read => "rx_fifo_read",
         readdata => "rx_fifo_readdata",
         write => "rx_fifo_write",
         writedata => "rx_fifo_writedata",
         empty => "rx_fifo_empty",
         # For error checking, might want to monitor rx fifo full.
         full => "open",
      },
    }),
    $fifo->make_instance({
      name => "tx_fifo",
      port_map => {
         read => "tx_fifo_read",
         readdata => "tx_fifo_readdata",
         write => "tx_fifo_write",
         writedata => "tx_fifo_writedata",
         # For error checking, might want to monitor tx fifo empty.
         empty => "open",
         full => "tx_fifo_full",
      },
    }),
  );
  my $sync = spi_slave::sync->new({});
  $module->add_contents(
    $sync->make_instance({}),
  );
  my $control = spi_slave::control->new({
    datawidth => $this->datawidth(),
  });
  $module->add_contents(
    $control->make_instance({}),
  );
  my $sink = spi_slave::av_st_sink->new({
    datawidth => $this->datawidth(),
  });
  $module->add_contents(
    # Nuisance, here: have to declare this signal so its width is 
    # correct at port-validation time.  Even without the declaration, 
    # the width is correct in the HDL.
    e_signal->new({name => "sink_data", width => $this->datawidth(),}),
    $sink->make_instance({}),
  );

  my $source = spi_slave::av_st_source->new({
    datawidth => $this->datawidth(),
  });
  $module->add_contents(
    # Nuisance, here: have to declare this signal so its width is 
    # correct at port-validation time.  Even without the declaration, 
    # the width is correct in the HDL.
    e_signal->new({name => "source_data", width => $this->datawidth(),}),
    $source->make_instance({}),
  );
}

sub get_fields
{
  my $class = shift;

  my %fields = (
    datawidth => { range => [1, undef], },
    lsbfirst => { range => [0, 1], },
  );

  return \%fields;
}

sub declare_ports
{
  my $this = shift;

  my %inputs = (
    clk => {
      width => 1,
    },
    reset_n => {
      width => 1,
    },
    SS_n => {
      width => 1,
    },
    MOSI => {
      width => 1,
    },
    SCLK => {
      width => 1,
    },
    sink_data => {
      width => $this->datawidth(),
    },
    sink_valid => {
      width => 1,
    },
    source_ready => {
      width => 1,
    },
  );
  my %outputs = (
    MISO => {
      width => 1,
    },
    sink_ready => {
      width => 1,
    },
    source_data => {
      width => $this->datawidth(),
    },
    source_valid => {
      width => 1,
    },
  );
  map {$inputs{$_}->{direction} = "input"} keys %inputs;
  map {$outputs{$_}->{direction} = "output"} keys %outputs;
  my %ports = (
    %inputs,
    %outputs,
  );

  return \%ports;
}

1;

